@@ -1,12 +1,14 @@
 import requests
 from .entity import (
     Aggs, Aggsv2, Aggsv2Set,
-    Trade, Trades,
-    Quote, Quotes,
+    Trade, Trades, TradesV2,
+    Quote, Quotes, QuotesV2,
     Exchange, SymbolTypeMap, ConditionMap,
-    Company, Dividends, Splits, Earnings, Financials, NewsList, Ticker
+    Company, Dividends, Splits, Earnings, Financials, NewsList, Ticker,
+    DailyOpenClose
 )
 from alpaca_trade_api.common import get_polygon_credentials
+from deprecated import deprecated
 
 
 def _is_list_like(o):
@@ -41,6 +43,10 @@ class REST(object):
         path = '/meta/symbol-types'
         return SymbolTypeMap(self.get(path))
 
+    @deprecated(
+        'historic_trades v1 is deprecated and will be removed from the ' +
+        'Polygon API in the future. Please upgrade to historic_trades_v2.'
+    )
     def historic_trades(self, symbol, date, offset=None, limit=None):
         path = '/historic/trades/{}/{}'.format(symbol, date)
         params = {}
@@ -52,6 +58,28 @@ class REST(object):
 
         return Trades(raw)
 
+    def historic_trades_v2(
+        self, symbol, date, timestamp=None, timestamp_limit=None,
+        reverse=None, limit=None
+    ):
+        path = '/ticks/stocks/trades/{}/{}'.format(symbol, date)
+        params = {}
+        if timestamp is not None:
+            params['timestamp'] = timestamp
+        if timestamp_limit is not None:
+            params['timestampLimit'] = timestamp_limit
+        if reverse is not None:
+            params['reverse'] = reverse
+        if limit is not None:
+            params['limit'] = limit
+        raw = self.get(path, params, 'v2')
+
+        return TradesV2(raw)
+
+    @deprecated(
+        'historic_quotes v1 is deprecated and will be removed from the ' +
+        'Polygon API in the future. Please upgrade to historic_quotes_v2.'
+    )
     def historic_quotes(self, symbol, date, offset=None, limit=None):
         path = '/historic/quotes/{}/{}'.format(symbol, date)
         params = {}
@@ -63,6 +91,24 @@ class REST(object):
 
         return Quotes(raw)
 
+    def historic_quotes_v2(
+        self, symbol, date, timestamp=None, timestamp_limit=None,
+        reverse=None, limit=None
+    ):
+        path = '/ticks/stocks/nbbo/{}/{}'.format(symbol, date)
+        params = {}
+        if timestamp is not None:
+            params['timestamp'] = timestamp
+        if timestamp_limit is not None:
+            params['timestampLimit'] = timestamp_limit
+        if reverse is not None:
+            params['reverse'] = reverse
+        if limit is not None:
+            params['limit'] = limit
+        raw = self.get(path, params, 'v2')
+
+        return QuotesV2(raw)
+
     def historic_agg(self, size, symbol,
                      _from=None, to=None, limit=None):
         path = '/historic/agg/{}/{}'.format(size, symbol)
@@ -96,6 +142,11 @@ class REST(object):
         raw = self.get(path, params, version='v2')
         return Aggsv2Set(raw)
 
+    def daily_open_close(self, symbol, date):
+        path = '/open-close/{}/{}'.format(symbol, date)
+        raw = self.get(path)
+        return DailyOpenClose(raw)
+
     def last_trade(self, symbol):
         path = '/last/stocks/{}'.format(symbol)
         raw = self.get(path)
@@ -107,6 +158,11 @@ class REST(object):
         # TODO status check
         return Quote(raw['last'])
 
+    def previous_day_bar(self, symbol):
+        path = '/aggs/ticker/{}/prev'.format(symbol)
+        raw = self.get(path, version='v2')
+        return Aggsv2(raw)
+
     def condition_map(self, ticktype='trades'):
         path = '/meta/conditions/{}'.format(ticktype)
         return ConditionMap(self.get(path))
