@@ -12,8 +12,10 @@ from .common import (
 from .entity import (
     Account, AccountConfigurations, AccountActivity,
     Asset, Order, Position, BarSet, Clock, Calendar,
+    Watchlist, PortfolioHistory
 )
 from . import polygon
+from . import alpha_vantage
 
 logger = logging.getLogger(__name__)
 
@@ -73,6 +75,7 @@ class REST(object):
             'APCA_RETRY_CODES', '429,504').split(',')]
         self.polygon = polygon.REST(
             self._key_id, 'staging' in self._base_url)
+        self.alpha_vantage = alpha_vantage.REST(self._key_id)
 
     def _request(
         self,
@@ -194,7 +197,7 @@ class REST(object):
         return AccountConfigurations(resp)
 
     def list_orders(self, status=None, limit=None, after=None, until=None,
-                    direction=None, params=None):
+                    direction=None, params=None, nested=None):
         '''
         Get a list of orders
         https://docs.alpaca.markets/web-api/orders/#get-a-list-of-orders
@@ -211,12 +214,16 @@ class REST(object):
             params['direction'] = direction
         if status is not None:
             params['status'] = status
-        resp = self.get('/orders', params)
+        if nested is not None:
+            params['nested'] = nested
+        url = '/orders'
+        resp = self.get(url, params)
         return [Order(o) for o in resp]
 
     def submit_order(self, symbol, qty, side, type, time_in_force,
                      limit_price=None, stop_price=None, client_order_id=None,
-                     extended_hours=None):
+                     extended_hours=None, order_class=None,
+                     take_profit=None, stop_loss=None):
         '''Request a new order'''
         params = {
             'symbol': symbol,
@@ -233,19 +240,27 @@ class REST(object):
             params['client_order_id'] = client_order_id
         if extended_hours is not None:
             params['extended_hours'] = extended_hours
+        if order_class is not None:
+            params['order_class'] = order_class
+        if take_profit is not None:
+            params['take_profit'] = take_profit
+        if stop_loss is not None:
+            params['stop_loss'] = stop_loss
         resp = self.post('/orders', params)
         return Order(resp)
 
     def get_order_by_client_order_id(self, client_order_id):
         '''Get an order by client order id'''
-        resp = self.get('/orders:by_client_order_id', {
+        params = {
             'client_order_id': client_order_id,
-        })
+        }
+        resp = self.get('/orders:by_client_order_id', params)
         return Order(resp)
 
     def get_order(self, order_id):
         '''Get an order'''
-        resp = self.get('/orders/{}'.format(order_id))
+        params = {}
+        resp = self.get('/orders/{}'.format(order_id), params)
         return Order(resp)
 
     def replace_order(
@@ -291,11 +306,13 @@ class REST(object):
 
     def close_position(self, symbol):
         '''Liquidates the position for the given symbol at market price'''
-        self.delete('/positions/{}'.format(symbol))
+        resp = self.delete('/positions/{}'.format(symbol))
+        return Order(resp)
 
     def close_all_positions(self):
         '''Liquidates all open positions at market price'''
-        self.delete('/positions')
+        resp = self.delete('/positions')
+        return [Order(o) for o in resp]
 
     def list_assets(self, status=None, asset_class=None):
         '''Get a list of assets'''
@@ -354,6 +371,7 @@ class REST(object):
         direction=None,
         date=None,
         page_size=None,
+        page_token=None
     ):
         url = '/account/activities'
         params = {}
@@ -371,6 +389,8 @@ class REST(object):
             params['date'] = date
         if page_size is not None:
             params['page_size'] = page_size
+        if page_token is not None:
+            params['page_token'] = page_token
         resp = self.get(url, data=params)
         return [AccountActivity(o) for o in resp]
 
@@ -382,3 +402,55 @@ class REST(object):
             params['end'] = end
         resp = self.get('/calendar', data=params)
         return [Calendar(o) for o in resp]
+
+    def get_watchlists(self):
+        resp = self.get('/watchlists')
+        return [Watchlist(o) for o in resp]
+
+    def get_watchlist(self, watchlist_id):
+        resp = self.get('/watchlists/{}'.format((watchlist_id)))
+        return Watchlist(resp)
+
+    def add_watchlist(self, watchlist_name):
+        resp = self.post('/watchlists', data=dict(name=watchlist_name))
+        return [Watchlist(o) for o in resp]
+
+    def add_to_watchlist(self, watchlist_id, symbol):
+        resp = self.post(
+            '/watchlists/{}'.format(watchlist_id), data=dict(symbol=symbol)
+        )
+        return Watchlist(resp)
+
+    def update_watchlist(self, watchlist_id, name=None, symbols=None):
+        params = {}
+        if name is not None:
+            params['name'] = name
+        if symbols is not None:
+            params['symbols'] = symbols
+        resp = self.patch('/watchlists/{}'.format(watchlist_id), data=params)
+        return Watchlist(resp)
+
+    def delete_watchlist(self, watchlist_id):
+        self.delete('/watchlists/{}'.format(watchlist_id))
+
+    def delete_from_watchlist(self, watchlist_id, symbol):
+        self.delete('/watchlists/{}/{}'.format(watchlist_id, symbol))
+
+    def get_portfolio_history(
+        self, date_start=None, date_end=None, period=None,
+        timeframe=None, extended_hours=None
+    ):
+        params = {}
+        if date_start is not None:
+            params['date_start'] = date_start
+        if date_end is not None:
+            params['date_end'] = date_end
+        if period is not None:
+            params['period'] = period
+        if timeframe is not None:
+            params['timeframe'] = timeframe
+        if extended_hours is not None:
+            params['extended_hours'] = extended_hours
+        return PortfolioHistory(
+            self.get('/account/portfolio/history', data=params)
+        )
