@@ -1,5 +1,6 @@
 import asyncio
 import json
+import os
 import re
 import websockets
 from .common import get_base_url, get_credentials
@@ -10,15 +11,24 @@ import logging
 
 class StreamConn(object):
     def __init__(self, key_id=None, secret_key=None, base_url=None):
-        self._key_id, self._secret_key = get_credentials(key_id, secret_key)
+        self._key_id, self._secret_key, _ = get_credentials(key_id, secret_key)
         base_url = re.sub(r'^http', 'ws', base_url or get_base_url())
         self._endpoint = base_url + '/stream'
         self._handlers = {}
         self._handler_symbols = {}
         self._base_url = base_url
+        self._streams = set([])
         self._ws = None
+        self._retry = int(os.environ.get('APCA_RETRY_MAX', 3))
+        self._retry_wait = int(os.environ.get('APCA_RETRY_WAIT', 3))
+        self._retries = 0
         self.polygon = None
-        self.loop = asyncio.get_event_loop()
+        try:
+            self.loop = asyncio.get_event_loop()
+        except websockets.WebSocketException as wse:
+            logging.warn(wse)
+            self.loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(self.loop)
 
     async def _connect(self):
         ws = await websockets.connect(self._endpoint)
@@ -39,12 +49,13 @@ class StreamConn(object):
                 ("Invalid Alpaca API credentials, Failed to authenticate: {}"
                     .format(msg))
             )
+        else:
+            self._retries = 0
 
         self._ws = ws
         await self._dispatch('authorized', msg)
 
         asyncio.ensure_future(self._consume_msg())
-        return ws
 
     async def _consume_msg(self):
         ws = self._ws
@@ -57,9 +68,10 @@ class StreamConn(object):
                 stream = msg.get('stream')
                 if stream is not None:
                     await self._dispatch(stream, msg)
-        finally:
-            await ws.close()
-            self._ws = None
+        except websockets.WebSocketException as wse:
+            logging.warn(wse)
+            await self.close()
+            asyncio.ensure_future(self._ensure_ws())
 
     async def _ensure_polygon(self):
         if self.polygon is not None:
@@ -68,17 +80,30 @@ class StreamConn(object):
         if 'staging' in self._base_url:
             key_id += '-staging'
         self.polygon = polygon.StreamConn(key_id)
-        self.polygon._handlers = self._handlers
-        self.polygon._handler_symbols = self._handler_symbols
+        self.polygon._handlers = self._handlers.copy()
+        self.polygon._handler_symbols = self._handler_symbols.copy()
         await self.polygon.connect()
 
     async def _ensure_ws(self):
         if self._ws is not None:
             return
-        self._ws = await self._connect()
+
+        while self._retries <= self._retry:
+            try:
+                await self._connect()
+                if self._streams:
+                    await self.subscribe(self._streams)
+                break
+            except websockets.WebSocketException as wse:
+                logging.warn(wse)
+                self._ws = None
+                self._retries += 1
+                await asyncio.sleep(self._retry_wait * self._retry)
+        else:
+            raise ConnectionError("Max Retries Exceeded")
 
     async def subscribe(self, channels):
-        '''Start subscribing channels.
+        '''Start subscribing to channels.
         If the necessary connection isn't open yet, it opens now.
         '''
         ws_channels = []
@@ -91,6 +116,7 @@ class StreamConn(object):
 
         if len(ws_channels) > 0:
             await self._ensure_ws()
+            self._streams |= set(ws_channels)
             await self._ws.send(json.dumps({
                 'action': 'listen',
                 'data': {
@@ -125,7 +151,7 @@ class StreamConn(object):
             await self.polygon.unsubscribe(polygon_channels)
 
     def run(self, initial_channels=[]):
-        '''Run forever and block until exception is rasised.
+        '''Run forever and block until exception is raised.
         initial_channels is the channels to start with.
         '''
         loop = self.loop
@@ -142,8 +168,10 @@ class StreamConn(object):
         '''Close any of open connections'''
         if self._ws is not None:
             await self._ws.close()
+            self._ws = None
         if self.polygon is not None:
             await self.polygon.close()
+            self.polygon = None
 
     def _cast(self, channel, msg):
         if channel == 'account_updates':
